// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef DISTANCE_FUNCTORS_H__RTG
#define DISTANCE_FUNCTORS_H__RTG

namespace RTG
{
    /// A functor for computing the Euclidean squared distance of two given points.
    template<typename _T>
	class EuclideanSqrdDistanceFunc
	{
	 public:
		EuclideanSqrdDistanceFunc(unsigned int dim)
		: dim_(dim)
		{}
		
		double operator()(const _T& data1, const _T& data2) const 
		{
			double res = 0;
			for (unsigned int i(0); i < dim_; ++i)
			{
				double diff = data1[i] - data2[i];
				res += diff * diff;
			}
			return res;
		}

	 private:
		unsigned int	dim_;

	}; //EuclideanSqrdDistanceFunc

	
    /// A functor for computing the Multi Robot squared distance of two given points, representing two configurations.
    /*!
        Each configuration represents the position of a set of robots.
        The distance according to multi-robot metric is the sum of distances 
        traveled by each robot.
        This functor computes the squared distance of two configurations.
    */
    template<typename _T>
	class MultiRobotSqrdDistanceFunc
	{
	 public:
		MultiRobotSqrdDistanceFunc(unsigned int num_of_robots, unsigned int dim = 3)
		: dim_(dim), num_of_robots_(num_of_robots)
		{}
		
		double operator()(const _T& data1, const _T& data2) const 
		{
			double res = 0;
			for (unsigned int i(0); i < num_of_robots_; ++i)
			{
				double res_i = 0;
				for (unsigned int j(0); j < dim_; ++j)
				{
					unsigned int coord_index = i * num_of_robots_ + j;
					double diff = data1[coord_index] - data2[coord_index];
					res_i += diff*diff;
				}
				res += sqrt (res_i);
			}
			return res*res;
		}

	 private:
		
		unsigned int dim_;
        unsigned int num_of_robots_;   

	}; //MultiRobotSqrdDistanceFunc

} //namespace RTG

#endif //DISTANCE_FUNCTORS_H__RTG